<?php

/**
 * Controller class for handling all requests related to courses.
 *
 * @author Softpro India Pvt. Ltd.
 */
defined('BASEPATH') OR exit('No direct script access allowed');

class Course extends CI_Controller {

    public function __construct() {
        parent::__construct();
        $this->load->model('admin/CourseManagement');
        $this->load->model('admin/BranchManagement');
        $this->load->model('admin/DepartmentManagement');
        $this->load->model('admin/UCSMappingManagement');
    }

    public function index() {
        if ($this->sessionvalidator->isLoggedIn() && $this->sessionvalidator->isAccessGranted()) {
            $allCourseInfo = array();
            $courseInfo = $this->CourseManagement->getAllCourses()->result();
            for ($i = 0; $i < sizeof($courseInfo); $i++) {
                $subCourseInfo = $this->CourseManagement->getAllSubCoursesBy($courseInfo[$i]->course_id)->result();
                $universityAndBranchInfo = $this->UCSMappingManagement->getUniversityAndBranchInfoBy($courseInfo[$i]->course_id)->result();
                if (sizeof($universityAndBranchInfo)) {
                    $universityAndBranch = $universityAndBranchInfo[0];
                    $recognitionTypeAndBody = ($courseInfo[$i]->course_approval_flag == "A") ? "Affiliated To " . stripslashes($universityAndBranch->univ_name) : "Recognized By " . stripslashes($universityAndBranch->univ_name);
                } else {
                    $recognitionTypeAndBody = ($courseInfo[$i]->course_approval_flag == "A") ? "Univ. Not Yet Updated" : "Recognition Not Yet Updated";
                }
                $timeMgmt = "";
                $regDuration = "";
                $latDuration = "";
                $isLateralAllowed = false;
                $lateralYears = "";
                $recognitionType = ($courseInfo[$i]->course_approval_flag == "A") ? "University Affiliation" : "Recognized By Any Governing Body Like UGC,AICTE Etc.";
                for ($j = 0; $j < sizeof($subCourseInfo); $j++) {
                    if ($subCourseInfo[$j]->tcsm_course_type == "Lateral Entry" && $subCourseInfo[$j]->tcsm_active_status == "T") {
                        $isLateralAllowed = true;
                        $lateralYears = $subCourseInfo[$j]->tcsm_duration;
                    }
                }
                if ($courseInfo[$i]->course_time_mgmt_flag == "S") {
                    $timeMgmt = "Semester Wise";
                    $regDuration = "<b>Reg.: </b>" . $courseInfo[$i]->course_duration . " Years / " . $courseInfo[$i]->course_no_of_sems . " Semesters";
                    $latDuration = ($isLateralAllowed) ? "<b>LE.:</b>" . $lateralYears . " Years / " . ($lateralYears * 2) . " Semesters" : "";
                } else if ($courseInfo[$i]->course_time_mgmt_flag == "Y") {
                    $timeMgmt = "Yearly";
                    $regDuration = "<b>Reg.: </b>" . $courseInfo[$i]->course_duration . " Years";
                    $latDuration = ($isLateralAllowed) ? "<b>LE.:</b>" . $lateralYears . " Years" : "";
                } else {
                    $timeMgmt = "Trimester Wise";
                    $regDuration = "<b>Reg.: </b>" . $courseInfo[$i]->course_duration . " Years / " . $courseInfo[$i]->course_no_of_sems . " Trimesters";
                    $latDuration = ($isLateralAllowed) ? "<b>LE.:</b>" . $lateralYears . " Years / " . ($lateralYears * 3) . " Trimesters" : "";
                }
                $thisCourseInfo = array(
                    'course_id' => $courseInfo[$i]->course_id,
                    'course_alias' => stripslashes($courseInfo[$i]->course_name),
                    'course_reg_duration' => $regDuration,
                    'course_lat_duration' => $latDuration,
                    'course_entry_type' => ($latDuration == "") ? "Regular Only" : "Regular & Lateral Both",
                    'course_category' => $courseInfo[$i]->course_category,
                    'course_active_status' => $courseInfo[$i]->course_active_status,
                    'course_delete_status' => $courseInfo[$i]->course_delete_status,
                    'course_added_on' => date("d-m-Y h:i:s A", strtotime($courseInfo[$i]->course_added_on)),
                    'course_updated_on' => date("d-m-Y h:i:s A", strtotime($courseInfo[$i]->course_updated_on)),
                    'addedByAdmin' => $courseInfo[$i]->addedByAdmin,
                    'updatedByAdmin' => $courseInfo[$i]->updatedByAdmin,
                    'course_sub_dept' => $courseInfo[$i]->sub_dept_name,
                    'course_dept' => $courseInfo[$i]->dept_name,
                    'course_time_mgmt_flag' => $timeMgmt,
                    'course_full_name' => stripslashes($courseInfo[$i]->course_full_name),
                    'course_time_type' => $courseInfo[$i]->course_time_type,
                    'course_recognition_type' => $recognitionType,
                    'univ_apprv_body_name' => $recognitionTypeAndBody
                );
                array_push($allCourseInfo, $thisCourseInfo);
            }
            $viewData['allCourses'] = $allCourseInfo;
            $this->load->view('admin/courses', $viewData);
        } else {
            redirect("admin/");
        }
    }

    public function createCourse() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $viewData['departments'] = $this->DepartmentManagement->getActiveNonDeletedDepartments()->result();
            $this->load->view('admin/createCourse', $viewData);
        } else {
            redirect("admin/");
        }
    }

    public function saveNewCourse() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $this->form_validation->set_rules('courseAlias', 'Course Alias', 'trim|required', array('required' => 'Course Alias Can Not Be Blank.'));
            $this->form_validation->set_rules('courseName', 'Course Full Name', 'trim|required', array('required' => 'Course Full Name Is Required.'));
            $this->form_validation->set_rules('courseLEAllowed', 'Lateral Entry Allowed', 'trim|required', array('required' => 'Please Select If Lateral Entry Is Allowed.'));
            $this->form_validation->set_rules('courseTimeMgmt', 'Time Management', 'trim|required', array('required' => 'Please Select Time Management Type.'));
            $this->form_validation->set_rules('courseDuration', 'Course Duration (In Years)', 'trim|required|numeric', array('required' => 'Please Specify Course Duration In Years.'));
            $this->form_validation->set_rules('courseSemTrims', 'No. Of Years/Sems./Trims.', 'trim|required|numeric', array('required' => 'No. Of Years/Sems/Trims Can Not Be Blank.'));
            $this->form_validation->set_rules('courseCategory', 'Course Category', 'trim|required', array('required' => 'Please Select Course Category.'));
            $this->form_validation->set_rules('courseRecognitionBody', 'Course Recognition Body', 'trim|required', array('required' => 'Please Select Course Recognition Body.'));
            $this->form_validation->set_rules('courseTimeType', 'Course Time Type', 'trim|required', array('required' => 'Please Specify Course Time Type.'));
            $this->form_validation->set_rules('courseDept', 'Course Department', 'trim|required', array('required' => 'Course Department Selection Is Required.'));
            $this->form_validation->set_rules('courseSubDept', 'Course Sub-Department', 'trim|required', array('required' => 'Course Sub-Department Selection Is Required.'));
            $queryByCourseName = $this->CourseManagement->getCourseByFullName(trim($this->input->post('courseName')));
            $courseInfoByName = $queryByCourseName->result();
            $queryByCourseAlias = $this->CourseManagement->getCourseByAlias(trim($this->input->post('courseAlias')));
            $courseInfoByAlias = $queryByCourseAlias->result();
            if ($this->form_validation->run() == FALSE) {
                $this->createCourse();
            } else if (sizeof($courseInfoByAlias)) {
                $this->session->set_flashdata('errorMessage', "A Course With This Alias (" . $this->input->post('courseAlias') . ") Already Exits. Please Choose A Different Alias.");
                $this->createCourse();
            } else if (sizeof($courseInfoByName)) {
                $this->session->set_flashdata('errorMessage', "A Course With This Name (" . $this->input->post('courseName') . ") Already Exits. Please Choose A Different Name.");
                $this->createCourse();
            } else {
                $newCourseInfo = array(
                    'course_name' => addslashes(trim($this->input->post('courseAlias'))),
                    'course_full_name' => addslashes(trim($this->input->post('courseName'))),
                    'course_duration' => trim($this->input->post('courseDuration')),
                    'course_no_of_sems' => ($this->input->post('courseSemTrims')),
                    'course_category' => $this->input->post('courseCategory'),
                    'course_time_mgmt_flag' => $this->input->post('courseTimeMgmt'),
                    'course_time_type' => $this->input->post('courseTimeType'),
                    'course_approval_flag' => $this->input->post('courseRecognitionBody'),
                    'sub_dept_id' => $this->input->post('courseSubDept'),
                    'course_added_on' => date("Y-m-d H:i:s"),
                    'course_added_by' => $this->session->userdata("adminData")["smember_id"],
                    'course_updated_on' => date("Y-m-d H:i:s"),
                    'course_updated_by' => $this->session->userdata("adminData")["smember_id"]
                );
                $subCourseForRegularArray = array(
                    'tcsm_course_type' => 'Regular',
                    'tcsm_sem_start_from' => 1,
                    'tcsm_duration' => trim($this->input->post('courseDuration')),
                    'course_id' => '',
                    'tcsm_active_status' => "T"
                );
                $subCourseForLateralArray = array(
                    'tcsm_course_type' => 'Lateral Entry',
                    'tcsm_sem_start_from' => 3,
                    'tcsm_duration' => $this->input->post('courseDuration') - 1,
                    'course_id' => '',
                    'tcsm_active_status' => $this->input->post('courseLEAllowed')
                );
                $this->db->trans_start();
                $course_id = $this->CourseManagement->createNewCourse($newCourseInfo);
                if ($course_id) {
                    $subCourseForRegularArray['course_id'] = $course_id;
                    $subCourseForLateralArray['course_id'] = $course_id;
                    $regSubCourseId = $this->CourseManagement->createNewSubCourse($subCourseForRegularArray);
                    $latSubCourseId = $this->CourseManagement->createNewSubCourse($subCourseForLateralArray);
                    $regUCSMapArray = array(
                        'branch_id' => '0',
                        'univ_id' => '0',
                        'sub_course_id' => $regSubCourseId,
                        'session_id' => '0',
                        'ucs_rg_le_active_status' => 'F',
                        'session_end_year' => '0',
                        'ucs_map_added_on' => date("Y-m-d H:i:s"),
                        'ucs_map_added_by' => $this->session->userdata("adminData")["smember_id"],
                        'ucs_map_updated_on' => date("Y-m-d H:i:s"),
                        'ucs_map_updated_by' => $this->session->userdata("adminData")["smember_id"]
                    );
                    $latUCSMapArray = array(
                        'branch_id' => '0',
                        'univ_id' => '0',
                        'sub_course_id' => $latSubCourseId,
                        'session_id' => '0',
                        'ucs_rg_le_active_status' => 'F',
                        'session_end_year' => '0',
                        'ucs_map_added_on' => date("Y-m-d H:i:s"),
                        'ucs_map_added_by' => $this->session->userdata("adminData")["smember_id"],
                        'ucs_map_updated_on' => date("Y-m-d H:i:s"),
                        'ucs_map_updated_by' => $this->session->userdata("adminData")["smember_id"]
                    );
                    $ucsMapArray = array($regUCSMapArray, $latUCSMapArray);
                    if ($this->UCSMappingManagement->createNewUCSMappingMulti($ucsMapArray)) {
                        $this->db->trans_complete();
                        $this->session->set_flashdata('successMessage', 'Course Created Successfully.');
                        redirect("admin/Course");
                    } else {
                        $this->session->set_flashdata('errorMessage', 'An Error Occured While Creating Course. Try Later.');
                        redirect("admin/Course");
                    }
                } else {
                    $this->session->set_flashdata('errorMessage', 'An Error Occured While Creating Course. Try Later.');
                    redirect("admin/Course");
                }
            }
        } else {
            redirect("admin/");
        }
    }

    public function editCourse($course_id) {
        if ($this->sessionvalidator->isLoggedIn()) {
            $viewData['departments'] = $this->DepartmentManagement->getActiveNonDeletedDepartments()->result();
            $viewData['courseInfo'] = $this->CourseManagement->getCoursesBy($course_id)->result()[0];
            $subCourseInfo = $this->CourseManagement->getAllSubCoursesBy($course_id)->result();
            for ($i = 0; $i < sizeof($subCourseInfo); $i++) {
                if ($subCourseInfo[$i]->tcsm_course_type == "Regular") {
                    $viewData['subCourseRegId'] = $subCourseInfo[$i]->tcsm_id;
                } else {
                    $viewData['subCourseLatId'] = $subCourseInfo[$i]->tcsm_id;
                }
                if ($subCourseInfo[$i]->tcsm_course_type == "Lateral Entry" && $subCourseInfo[$i]->tcsm_active_status == 'T') {
                    $viewData['isLatAllowed'] = true;
                } else {
                    $viewData['isLatAllowed'] = false;
                }
            }
            $this->load->view('admin/editCourse', $viewData);
        } else {
            redirect("admin/");
        }
    }

    public function updateCourse() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $course_id = $this->input->post('courseId');
            $sub_course_reg_id = $this->input->post('subCourseRegId');
            $sub_course_lat_id = $this->input->post('subCourseLatId');
            $this->form_validation->set_rules('courseAlias', 'Course Alias', 'trim|required', array('required' => 'Course Alias Can Not Be Blank.'));
            $this->form_validation->set_rules('courseName', 'Course Full Name', 'trim|required', array('required' => 'Course Full Name Is Required.'));
            $this->form_validation->set_rules('courseLEAllowed', 'Lateral Entry Allowed', 'trim|required', array('required' => 'Please Select If Lateral Entry Is Allowed.'));
            $this->form_validation->set_rules('courseTimeMgmt', 'Time Management', 'trim|required', array('required' => 'Please Select Time Management Type.'));
            $this->form_validation->set_rules('courseDuration', 'Course Duration (In Years)', 'trim|required|numeric', array('required' => 'Please Specify Course Duration In Years.'));
            $this->form_validation->set_rules('courseSemTrims', 'No. Of Years/Sems./Trims.', 'trim|required|numeric', array('required' => 'No. Of Years/Sems/Trims Can Not Be Blank.'));
            $this->form_validation->set_rules('courseCategory', 'Course Category', 'trim|required', array('required' => 'Please Select Course Category.'));
            $this->form_validation->set_rules('courseRecognitionBody', 'Course Recognition Body', 'trim|required', array('required' => 'Please Select Course Recognition Body.'));
            $this->form_validation->set_rules('courseTimeType', 'Course Time Type', 'trim|required', array('required' => 'Please Specify Course Time Type.'));
            $this->form_validation->set_rules('courseDept', 'Course Department', 'trim|required', array('required' => 'Course Department Selection Is Required.'));
            $this->form_validation->set_rules('courseSubDept', 'Course Sub-Department', 'trim|required', array('required' => 'Course Sub-Department Selection Is Required.'));
            if ($this->form_validation->run() == FALSE) {
                $this->editCourse($course_id);
            } else if (!$this->CourseManagement->isCourseAliasSafeUpdate($course_id, trim($this->input->post('courseAlias')))) {
                $this->session->set_flashdata('errorMessage', "A Course With This Alias (" . $this->input->post('courseAlias') . ") Already Exits. Please Choose A Different Alias.");
                $this->editCourse($course_id);
            } else if (!$this->CourseManagement->isCourseNameSafeUpdate($course_id, trim($this->input->post('courseName')))) {
                $this->session->set_flashdata('errorMessage', "A Course With This Name (" . $this->input->post('courseName') . ") Already Exits. Please Choose A Different Name.");
                $this->editCourse($course_id);
            } else {
                $courseInfo = array(
                    'course_id' => $course_id,
                    'course_name' => addslashes(trim($this->input->post('courseAlias'))),
                    'course_full_name' => addslashes(trim($this->input->post('courseName'))),
                    'course_duration' => trim($this->input->post('courseDuration')),
                    'course_no_of_sems' => ($this->input->post('courseSemTrims')),
                    'course_category' => $this->input->post('courseCategory'),
                    'course_time_mgmt_flag' => $this->input->post('courseTimeMgmt'),
                    'course_time_type' => $this->input->post('courseTimeType'),
                    'course_approval_flag' => $this->input->post('courseRecognitionBody'),
                    'sub_dept_id' => $this->input->post('courseSubDept'),
                    'course_updated_on' => date("Y-m-d H:i:s"),
                    'course_updated_by' => $this->session->userdata("adminData")["smember_id"]
                );
                $subCourseForRegularArray = array(
                    'tcsm_sem_start_from' => 1,
                    'tcsm_duration' => trim($this->input->post('courseDuration')),
                    'tcsm_id' => $sub_course_reg_id,
                    'tcsm_active_status' => "T"
                );
                $subCourseForLateralArray = array(
                    'tcsm_sem_start_from' => 3,
                    'tcsm_duration' => $this->input->post('courseDuration') - 1,
                    'tcsm_id' => $sub_course_lat_id,
                    'tcsm_active_status' => $this->input->post('courseLEAllowed')
                );
                $this->db->trans_start();
                if ($this->CourseManagement->updateCourseInfo($courseInfo)) {
                    if ($this->CourseManagement->updateSubCourseInfo($subCourseForRegularArray) && $this->CourseManagement->updateSubCourseInfo($subCourseForLateralArray)) {
                        $this->db->trans_complete();
                        $this->session->set_flashdata('successMessage', 'Course Updated Successfully.');
                        redirect("admin/Course");
                    } else {
                        $this->session->set_flashdata('errorMessage', 'An Error Occured While Updating Course. Try Later.');
                        redirect("admin/Course");
                    }
                } else {
                    $this->session->set_flashdata('errorMessage', 'An Error Occured While Updating Course. Try Later.');
                    redirect("admin/Course");
                }
            }
        } else {
            redirect("admin/");
        }
    }

    public function toggleCourseActiveStatus($course_id, $toUpdateStatus) {
        if ($this->sessionvalidator->isLoggedIn()) {
            $courseUpdateData = array(
                'course_id' => $course_id,
                'course_updated_on' => date("Y-m-d H:i:s"),
                'course_updated_by' => $this->session->userdata("adminData")["smember_id"],
                'course_active_status' => $toUpdateStatus
            );
            if ($this->CourseManagement->updateCourseInfo($courseUpdateData)) {
                $this->session->set_flashdata('successMessage', 'Course Status Updated Successfully.');
                redirect("admin/Course");
            } else {
                $this->session->set_flashdata('errorMessage', 'Some Error Occurred While Updating Course Status. Try Later.');
                redirect(current_url());
            }
        } else {
            redirect("admin/");
        }
    }

    public function deleteCourse($course_id) {
        if ($this->sessionvalidator->isLoggedIn()) {
            $courseUpdateData = array(
                'course_id' => $course_id,
                'course_updated_on' => date("Y-m-d H:i:s"),
                'course_updated_by' => $this->session->userdata("adminData")["smember_id"],
                'course_delete_status' => 'T'
            );
            if ($this->CourseManagement->updateCourseInfo($courseUpdateData)) {
                $this->session->set_flashdata('successMessage', 'Course Deleted Successfully.');
                redirect("admin/Course");
            } else {
                $this->session->set_flashdata('errorMessage', 'Some Error Occurred While Deleting Course. Try Later.');
                redirect(current_url());
            }
        } else {
            redirect("admin/");
        }
    }

    public function undoDeleteCourse($course_id) {
        if ($this->sessionvalidator->isLoggedIn()) {
            $courseUpdateData = array(
                'course_id' => $course_id,
                'course_updated_on' => date("Y-m-d H:i:s"),
                'course_updated_by' => $this->session->userdata("adminData")["smember_id"],
                'course_delete_status' => 'F'
            );
            if ($this->CourseManagement->updateCourseInfo($courseUpdateData)) {
                $this->session->set_flashdata('successMessage', 'Course Recovered Successfully.');
                redirect("admin/Course");
            } else {
                $this->session->set_flashdata('errorMessage', 'Some Error Occurred While Recovering Course. Try Later.');
                redirect(current_url());
            }
        } else {
            redirect("admin/");
        }
    }

    /* Request Handlers For Course Profile */

    public function courseProfiles() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $viewData['courseProfiles'] = $this->CourseManagement->getAllCourseProfile()->result();
            $this->load->view('admin/academics/courseProfiles', $viewData);
        } else {
            redirect("admin/");
        }
    }

    public function createCourseProfile() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $viewData['branches'] = $this->BranchManagement->getAllNonDeletedActiveBrances()->result();
            $viewData['courses'] = $this->CourseManagement->getNonDeletedCourses()->result();
            $this->load->view('admin/academics/createCourseProfile', $viewData);
        } else {
            redirect("admin/");
        }
    }

    public function saveNewCourseProfile() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $this->form_validation->set_rules('course', 'Course', 'trim|required', array('required' => 'Please Select Any Course.'));
            $this->form_validation->set_rules('session', 'Session', 'trim|required', array('required' => 'Please Select Any Session.'));
            $this->form_validation->set_rules('entryType', 'Entry Type', 'trim|required', array('required' => 'Please Select Any Entry Type.'));
            $this->form_validation->set_rules('semOrYear', 'Semester Or Year', 'trim|required', array('required' => 'Please Select Semester Or Year.'));
            $queryByEntryTypeAndSession = $this->CourseManagement->getCourseProfileByCourseTypeAndSession(trim($this->input->post('entryType')), trim($this->input->post('session')), trim($this->input->post('semOrYear')));
            $courseProfileInfoByEntryTypeAndSession = $queryByEntryTypeAndSession->result();
            if ($this->form_validation->run() == FALSE) {
                $this->createCourseProfile();
            } else if (empty($_FILES['courseProfileFile']['name'])) {
                $this->form_validation->set_rules('courseProfileFile', 'Course Profile File', 'required', array('required' => 'Course Profile File Is Required.'));
                $this->form_validation->run();
                $this->createCourseProfile();
            } else if (sizeof($courseProfileInfoByEntryTypeAndSession)) {
                $this->session->set_flashdata('errorMessage', "Course Profile For Selected Course/Entry Type & Session Already Exits. Please Choose A Different Course/Entry Type & Session.");
                $this->createCourseProfile();
            } else {
                $courseProfileFileWithPath = "";
                if (!empty($_FILES['courseProfileFile']['name'])) {
                    $courseProfileFile['upload_path'] = './assets/admin/uploads/courseProfiles/';
                    $courseProfileFile['allowed_types'] = 'pdf';
                    $courseProfileFile['max_size'] = '2048';
                    $courseProfileFile['file_ext_tolower'] = TRUE;
                    $courseProfileFile['encrypt_name'] = TRUE;
                    $this->load->library('upload', $courseProfileFile);
                    if (!$this->upload->do_upload('courseProfileFile')) {
                        $this->session->set_flashdata('errorMessage', "Course Profile File Can Not Be Uploaded As It Violates Upload File Criteria. File Size 2 MB Max. & Allowed Extension: Only *.pdf.");
                        $this->createCourseProfile();
                    } else {
                        $courseProfileFileWithPath = "/assets/admin/uploads/courseProfiles/" . $this->upload->data()['file_name'];
                    }
                }
                $newCourseProfileInfo = array(
                    'course_id' => trim($this->input->post("course")),
                    'session_id' => trim($this->input->post("session")),
                    'ucs_map_id' => trim($this->input->post("entryType")),
                    'cp_sem_or_year' => trim($this->input->post("semOrYear")),
                    'cp_file_path' => $courseProfileFileWithPath,
                    'cp_added_by' => $this->session->userdata("adminData")["smember_id"],
                    'cp_added_on' => date("Y-m-d H:i:s"),
                    'cp_updated_by' => $this->session->userdata("adminData")["smember_id"],
                    'cp_updated_on' => date("Y-m-d H:i:s")
                );
                if ($this->CourseManagement->createNewCourseProfile($newCourseProfileInfo)) {
                    $this->session->set_flashdata('successMessage', 'Course Profile Created Successfully.');
                    redirect("admin/Course/courseProfiles");
                } else {
                    $this->session->set_flashdata('successMessage', 'An Error Occured While Saving Course Profile. Try Again.');
                    redirect(current_url());
                }
            }
        } else {
            redirect("admin/");
        }
    }

    public function editCourseProfile($cp_id) {
        if ($this->sessionvalidator->isLoggedIn()) {
            $viewData['branches'] = $this->BranchManagement->getAllNonDeletedActiveBrances()->result();
            $viewData['courses'] = $this->CourseManagement->getNonDeletedCourses()->result();
            $viewData['coursesProfileInfo'] = $this->CourseManagement->getCourseProfileInfoBy($cp_id)->result()[0];
            $this->load->view('admin/academics/editCourseProfile', $viewData);
        } else {
            redirect("admin/");
        }
    }

    public function updateCourseProfile() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $courseProfileCurrentFileName = substr($this->input->post('currentCourseProfileFile'), strripos($this->input->post('currentCourseProfileFile'), "/") + 1);
            $courseProfileFileWithPath = "";
            if (!empty($_FILES['courseProfileFile']['name'])) {
                $courseProfileFile['upload_path'] = './assets/admin/uploads/courseProfiles/';
                $courseProfileFile['allowed_types'] = 'pdf';
                $courseProfileFile['max_size'] = '2048';
                $courseProfileFile['file_ext_tolower'] = TRUE;
                $courseProfileFile['encrypt_name'] = TRUE;
                $this->load->library('upload', $courseProfileFile);
                if (!$this->upload->do_upload('courseProfileFile')) {
                    $this->session->set_flashdata('errorMessage', "Course Profile File Can Not Be Uploaded As It Violates Upload File Criteria. File Size 2 MB Max. & Allowed Extension: Only *.pdf.");
                    $this->createCourseProfile();
                } else {
                    if (file_exists("./assets/admin/uploads/courseProfiles/" . $courseProfileCurrentFileName)) {
                        unlink("./assets/admin/uploads/courseProfiles/" . $courseProfileCurrentFileName);
                    }
                    $courseProfileFileWithPath = "/assets/admin/uploads/courseProfiles/" . $this->upload->data()['file_name'];
                }
            } else {
                $courseProfileFileWithPath = trim($this->input->post("currentCourseProfileFile"));
            }
            $courseProfileUpdateInfo = array(
                'cp_id' => trim($this->input->post("cp_id")),
                'cp_file_path' => $courseProfileFileWithPath,
                'cp_updated_by' => $this->session->userdata("adminData")["smember_id"],
                'cp_updated_on' => date("Y-m-d H:i:s")
            );
            if ($this->CourseManagement->updateCourseProfileInfo($courseProfileUpdateInfo)) {
                $this->session->set_flashdata('successMessage', 'Course Profile Updated Successfully.');
                redirect("admin/Course/courseProfiles");
            } else {
                $this->session->set_flashdata('successMessage', 'An Error Occured While Updating Course Profile. Try Again.');
                redirect(current_url());
            }
        } else {
            redirect("admin/");
        }
    }

    public function toggleCourseProfileActiveStatus($cp_id, $toUpdateStatus) {
        if ($this->sessionvalidator->isLoggedIn()) {
            $courseProfileUpdateData = array(
                'cp_id' => $cp_id,
                'cp_updated_on' => date("Y-m-d H:i:s"),
                'cp_updated_by' => $this->session->userdata("adminData")["smember_id"],
                'cp_active_status' => $toUpdateStatus
            );
            if ($this->CourseManagement->updateCourseProfileInfo($courseProfileUpdateData)) {
                $this->session->set_flashdata('successMessage', 'Course Profile Status Updated Successfully.');
                redirect("admin/Course/courseProfiles");
            } else {
                $this->session->set_flashdata('errorMessage', 'Some Error Occurred While Updating Course Profile Status. Try Later.');
                redirect(current_url());
            }
        } else {
            redirect("admin/");
        }
    }

    public function deleteCourseProfile($cp_id) {
        if ($this->sessionvalidator->isLoggedIn()) {
            $courseProfileUpdateData = array(
                'cp_id' => $cp_id,
                'cp_updated_on' => date("Y-m-d H:i:s"),
                'cp_updated_by' => $this->session->userdata("adminData")["smember_id"],
                'cp_delete_status' => 'T'
            );
            if ($this->CourseManagement->updateCourseProfileInfo($courseProfileUpdateData)) {
                $this->session->set_flashdata('successMessage', 'Course Profile Deleted Successfully.');
                redirect("admin/Course/courseProfiles");
            } else {
                $this->session->set_flashdata('errorMessage', 'Some Error Occurred While Deleting Course Profile. Try Later.');
                redirect(current_url());
            }
        } else {
            redirect("admin/");
        }
    }

    public function undoDeleteCourseProfile($cp_id) {
        if ($this->sessionvalidator->isLoggedIn()) {
            $courseProfileUpdateData = array(
                'cp_id' => $cp_id,
                'cp_updated_on' => date("Y-m-d H:i:s"),
                'cp_updated_by' => $this->session->userdata("adminData")["smember_id"],
                'cp_delete_status' => 'F'
            );
            if ($this->CourseManagement->updateCourseProfileInfo($courseProfileUpdateData)) {
                $this->session->set_flashdata('successMessage', 'Course Profile Recovered Successfully.');
                redirect("admin/Course/courseProfiles");
            } else {
                $this->session->set_flashdata('errorMessage', 'Some Error Occurred While Recovering Course Profile. Try Later.');
                redirect(current_url());
            }
        } else {
            redirect("admin/");
        }
    }

    /* Request Handlers For Course Profile */

    public function getEntryTypesOfGivenCourseSessionAndBranch() {
        $branch_id = $_POST['branch_id'];
        $course_id = $_POST['course_id'];
        $session_id = $_POST['session_id'];
        $query = $this->UCSMappingManagement->getEntryTypesInfoBy($branch_id, $session_id, $course_id);
        $allEntryTypeList = $query->result();
        if (sizeof($allEntryTypeList)) {
            $options = "<option value=''>Select Entry Type</option>";
            for ($i = 0; $i < (sizeof($allEntryTypeList)); $i++) {
                if ($allEntryTypeList[$i]->tcsm_course_type == "Lateral Entry") {
                    if ($allEntryTypeList[$i]->ucs_rg_le_active_status == 'T') {
                        $options .= "<option value=" . $allEntryTypeList[$i]->ucs_map_id . " selected>" . $allEntryTypeList[$i]->tcsm_course_type . "</option>";
                    } else {
                        $options .= "<option value=" . $allEntryTypeList[$i]->ucs_map_id . " disabled>" . $allEntryTypeList[$i]->tcsm_course_type . " (This Entry Type Is Not Allowed In The Selected Course/Session)</option>";
                    }
                } else {
                    if ($allEntryTypeList[$i]->ucs_rg_le_active_status == 'T') {
                        $options .= "<option value=" . $allEntryTypeList[$i]->ucs_map_id . " selected>" . $allEntryTypeList[$i]->tcsm_course_type . "</option>";
                    } else {
                        $options .= "<option value=" . $allEntryTypeList[$i]->ucs_map_id . " disabled>" . $allEntryTypeList[$i]->tcsm_course_type . " (This Entry Type Is Not Allowed In The Selected Course/Session)</option>";
                    }
                }
            }
            $responseData = array(
                'csrfName' => $this->security->get_csrf_token_name(),
                'csrfHash' => $this->security->get_csrf_hash(),
                'entry_types' => $options
            );
        } else {
            $options = "<option value=''>No Entry Type Available</option>";
            $responseData = array(
                'csrfName' => $this->security->get_csrf_token_name(),
                'csrfHash' => $this->security->get_csrf_hash(),
                'entry_types' => $options
            );
        }
        echo json_encode($responseData);
    }

    public function getEntryTypesOfGivenCourseSessionAndBranchSelected() {
        $branch_id = $_POST['branch_id'];
        $course_id = $_POST['course_id'];
        $session_id = $_POST['session_id'];
        $ucs_map_id = $_POST['ucs_map_id'];
        $query = $this->UCSMappingManagement->getEntryTypesInfoBy($branch_id, $session_id, $course_id);
        $allEntryTypeList = $query->result();
        if (sizeof($allEntryTypeList)) {
            $options = "<option value=''>Select Entry Type</option>";
            for ($i = 0; $i < (sizeof($allEntryTypeList)); $i++) {
                $selected = ($ucs_map_id == $allEntryTypeList[$i]->ucs_map_id) ? "selected" : "";
                if ($allEntryTypeList[$i]->tcsm_course_type == "Lateral Entry") {
                    if ($allEntryTypeList[$i]->ucs_rg_le_active_status == 'T') {
                        $options .= "<option value=" . $allEntryTypeList[$i]->ucs_map_id . " " . $selected . ">" . $allEntryTypeList[$i]->tcsm_course_type . "</option>";
                    } else {
                        $options .= "<option value=" . $allEntryTypeList[$i]->ucs_map_id . " " . $selected . " disabled>" . $allEntryTypeList[$i]->tcsm_course_type . " (This Entry Type Is Not Allowed In The Selected Course/Session)</option>";
                    }
                } else {
                    if ($allEntryTypeList[$i]->ucs_rg_le_active_status == 'T') {
                        $options .= "<option value=" . $allEntryTypeList[$i]->ucs_map_id . " " . $selected . ">" . $allEntryTypeList[$i]->tcsm_course_type . "</option>";
                    } else {
                        $options .= "<option value=" . $allEntryTypeList[$i]->ucs_map_id . " " . $selected . " disabled>" . $allEntryTypeList[$i]->tcsm_course_type . " (This Entry Type Is Not Allowed In The Selected Course/Session)</option>";
                    }
                }
            }
            $responseData = array(
                'csrfName' => $this->security->get_csrf_token_name(),
                'csrfHash' => $this->security->get_csrf_hash(),
                'entry_types' => $options
            );
        } else {
            $options = "<option value=''>No Entry Type Available</option>";
            $responseData = array(
                'csrfName' => $this->security->get_csrf_token_name(),
                'csrfHash' => $this->security->get_csrf_hash(),
                'entry_types' => $options
            );
        }
        echo json_encode($responseData);
    }

    public function getCourseFullInfo() {
        $course_id = $_POST['course_id'];
        $course_info = $this->CourseManagement->getCoursesBy($course_id)->result()[0];
        $courseTimeMgmtFlag = "";
        $courseApprovalFlag = "";
        if ($course_info->course_time_mgmt_flag == "S") {
            $courseTimeMgmtFlag = "Semester";
        } else if ($course_info->course_time_mgmt_flag == "Y") {
            $courseTimeMgmtFlag = "Year";
        } else {
            $courseTimeMgmtFlag = "Trimester";
        }
        if ($course_info->course_approval_flag == "R") {
            $courseApprovalFlag = "Approval By Any Governing Body Like UGC, AICTE Etc.";
        } else {
            $courseApprovalFlag = "University Affiliated";
        }
        $responseData = array(
            'csrfName' => $this->security->get_csrf_token_name(),
            'csrfHash' => $this->security->get_csrf_hash(),
            'course_id' => $course_info->ven_id,
            'course_name' => stripslashes($course_info->ven_owner_name),
            'course_full_name' => stripslashes($course_info->ven_business_name),
            'course_duration' => $course_info->ven_mobile,
            'course_no_of_sems' => $course_info->ven_email,
            'course_category' => $course_info->ven_gstin_no,
            'course_time_mgmt_flag' => $courseTimeMgmtFlag,
            'course_approval_flag' => $courseApprovalFlag,
            'course_time_type' => $course_info->course_time_type,
            'course_sub_dept' => stripslashes($course_info->sub_dept_name),
            'course_dept' => stripslashes($course_info->dept_name),
            'course_active_status' => ($course_info->course_active_status == "T") ? "<i class='fa fa-check' style='color:#00FF00;'></i> Active" : "<i class='fa fa-ban' style='color:#FF0000;'></i> Disabled",
            'course_delete_status' => ($course_info->course_delete_status == "T") ? "<i class='fa fa-trash' style='color:#FF0000;'></i> Deleted" : "Not Deleted",
            'course_added_by' => $course_info->addedByAdmin . " At " . date('d-m-Y h:i:s A', strtotime($course_info->course_added_on)),
            'course_updated_by' => $course_info->updatedByAdmin . " At " . date('d-m-Y h:i:s A', strtotime($course_info->course_updated_on))
        );
        echo json_encode($responseData);
    }

    public function getCourseSemesterOrYearForDropDown() {
        $course_id = $_POST['course_id'];
        $ucs_map_id = (isset($_POST['ucs_map_id'])) ? $_POST['ucs_map_id'] : "";
        $course_info = $this->CourseManagement->getCoursesBy($course_id)->result();
        if ($ucs_map_id != "") {
            $semesterStart = $this->UCSMappingManagement->getUCSMapInfoById($ucs_map_id)->result()[0]->tcsm_sem_start_from;
        } else {
            $semesterStart = 1;
        }
        if (sizeof($course_info)) {
            $thisCourseInfo = $course_info[0];
            if ($thisCourseInfo->course_time_mgmt_flag == "S") {
                $options = "<option value=''>Select Semester</option>";
                for ($i = $semesterStart; $i <= $thisCourseInfo->course_no_of_sems; $i++) {
                    $options .= "<option value=" . $i . ">" . $i . " Semester</option>";
                }
            } else if ($thisCourseInfo->course_time_mgmt_flag == "Y") {
                $options = "<option value=''>Select Year</option>";
                for ($i = $semesterStart; $i <= $thisCourseInfo->course_no_of_sems; $i++) {
                    $options .= "<option value=" . $i . ">" . $i . " Year</option>";
                }
            } else {
                $options = "<option value=''>Select Trimester</option>";
                for ($i = $semesterStart; $i <= $thisCourseInfo->course_no_of_sems; $i++) {
                    $options .= "<option value=" . $i . ">" . $i . " Trimester</option>";
                }
            }
        } else {
            $options = "<option value=''>This Course Is No Longer Available</option>";
        }

        $responseData = array(
            'csrfName' => $this->security->get_csrf_token_name(),
            'csrfHash' => $this->security->get_csrf_hash(),
            'sem_year_list' => $options
        );
        echo json_encode($responseData);
    }

    public function getCourseSemesterOrYearForDropDownSelected() {
        $course_id = $_POST['course_id'];
        $semester = $_POST['semester'];
        $ucs_map_id = (isset($_POST['ucs_map_id'])) ? $_POST['ucs_map_id'] : "";
        $course_info = $this->CourseManagement->getCoursesBy($course_id)->result();
        if ($ucs_map_id != "") {
            $semesterStart = $this->UCSMappingManagement->getUCSMapInfoById($ucs_map_id)->result()[0]->tcsm_sem_start_from;
        } else {
            $semesterStart = 1;
        }
        if (sizeof($course_info)) {
            $thisCourseInfo = $course_info[0];
            if ($thisCourseInfo->course_time_mgmt_flag == "S") {
                $options = "<option value=''>Select Semester</option>";
                for ($i = $semesterStart; $i <= $thisCourseInfo->course_no_of_sems; $i++) {
                    $selected = ($i == $semester) ? "selected" : "";
                    $options .= "<option value=" . $i . " " . $selected . " >" . $i . " Semester</option>";
                }
            } else if ($thisCourseInfo->course_time_mgmt_flag == "Y") {
                $options = "<option value=''>Select Year</option>";
                for ($i = $semesterStart; $i <= $thisCourseInfo->course_no_of_sems; $i++) {
                    $selected = ($i == $semester) ? "selected" : "";
                    $options .= "<option value=" . $i . " " . $selected . ">" . $i . " Year</option>";
                }
            } else {
                $options = "<option value=''>Select Trimester</option>";
                for ($i = $semesterStart; $i <= $thisCourseInfo->course_no_of_sems; $i++) {
                    $selected = ($i == $semester) ? "selected" : "";
                    $options .= "<option value=" . $i . " " . $selected . ">" . $i . " Trimester</option>";
                }
            }
        } else {
            $options = "<option value=''>This Course Is No Longer Available</option>";
        }

        $responseData = array(
            'csrfName' => $this->security->get_csrf_token_name(),
            'csrfHash' => $this->security->get_csrf_hash(),
            'sem_year_list' => $options
        );
        echo json_encode($responseData);
    }

    public function getCourseDropdownUsingSession() {
        $course_id = $_POST['course_id'];
        $session_id = $_POST['session_id'];
        $query = $this->UCSMappingManagement->getCoursesUsingSession($session_id);
        $courses = $query->result();
        if (sizeof($courses)) {
            $options = "<option value=''>Select Course</option>";
            for ($i = 0; $i < (sizeof($courses)); $i++) {
                $selected = ($course_id == $courses[$i]->course_id) ? "selected" : "";
                $options .= "<option value=" . $courses[$i]->course_id . " " . $selected . ">" . $courses[$i]->course_name . "</option>";
            }
            $responseData = array(
                'csrfName' => $this->security->get_csrf_token_name(),
                'csrfHash' => $this->security->get_csrf_hash(),
                'courses' => $options
            );
        } else {
            $options = "<option value=''>No Course Available</option>";
            $responseData = array(
                'csrfName' => $this->security->get_csrf_token_name(),
                'csrfHash' => $this->security->get_csrf_hash(),
                'courses' => $options
            );
        }
        echo json_encode($responseData);
    }

    /* Request Handlers For Course Wise Admission Closure For Admission Portal */

    public function counsellingClosure() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $viewData['courseClosureInfo'] = $this->CourseManagement->getAllCourseClosureInfo()->result();
            $this->load->view("admin/counsellingClosure", $viewData);
        } else {
            redirect("admin/");
        }
    }

    public function createCounsellingClosure() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $viewData['courses'] = $this->CourseManagement->getNonDeletedCourses()->result();
            $this->load->view("admin/createCounsellingClosure", $viewData);
        } else {
            redirect("admin/");
        }
    }

    public function saveNewCounsellingClosure() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $this->form_validation->set_rules('course', 'Course', 'trim|required', array('required' => 'Please Select Any Course.'));
            $this->form_validation->set_rules('scheduleStartDate', 'Start Date', 'trim|required', array('required' => 'Enter/Choose Start Date.'));
            $this->form_validation->set_rules('scheduleEndDate', 'End Date', 'trim|required', array('required' => 'Enter/Choose End Date.'));
            $queryByCourse = $this->CourseManagement->getCounsellingClosureInfo(trim($this->input->post('course')));
            $courseInfoByName = $queryByCourse->result();
            if ($this->form_validation->run() == FALSE) {
                $this->createCounsellingClosure();
            } else if (sizeof($courseInfoByName)) {
                $this->session->set_flashdata('errorMessage', "Counselling Clouse Date Already Exits For The Selected Course. If You Need Any Modificatin Use Edit.");
                $this->createCounsellingClosure();
            } else {
                $courseInfo = $this->CourseManagement->getCoursesBy(trim($this->input->post('course')))->result();
                $courseAlias = (sizeof($courseInfo)) ? $courseInfo[0]->course_name : "";
                $newCounsellingClosure = array(
                    'course_id' => trim($this->input->post('course')),
                    'aar_course' => stripslashes($courseAlias),
                    'aar_start_date' => date("Y-m-d", strtotime(str_replace('/', '-', trim($this->input->post("scheduleStartDate"))))),
                    'aar_end_date' => date("Y-m-d", strtotime(str_replace('/', '-', trim($this->input->post("scheduleEndDate"))))),
                    'aar_added_by' => $this->session->userdata("adminData")["smember_id"],
                    'aar_added_on' => date("Y-m-d H:i:s"),
                    'aar_updated_by' => $this->session->userdata("adminData")["smember_id"],
                    'aar_updated_on' => date("Y-m-d H:i:s")
                );
                if ($this->CourseManagement->createNewCounsellingClosure($newCounsellingClosure)) {
                    $this->session->set_flashdata('successMessage', 'Counselling Closure Created Successfully For Selected Course.');
                    redirect("admin/Course/counsellingClosure");
                } else {
                    $this->session->set_flashdata('errorMessage', 'An Error Occured While Creating Counselling Closure For Selected Course. Try Later.');
                    redirect(current_url());
                }
            }
        } else {
            redirect("admin/");
        }
    }

    public function toggleCounsellingClosureStatus($aar_id, $toUpdateStatus) {
        if ($this->sessionvalidator->isLoggedIn()) {
            $counsellingClosureData = array(
                'aar_id' => $aar_id,
                'aar_updated_on' => date("Y-m-d H:i:s"),
                'aar_updated_by' => $this->session->userdata("adminData")["smember_id"],
                'aar_status' => $toUpdateStatus
            );
            if ($this->CourseManagement->updateCounsellingClosureInfo($counsellingClosureData)) {
                $this->session->set_flashdata('successMessage', 'Counselling Closure Status Updated Successfully.');
                redirect("admin/Course/counsellingClosure");
            } else {
                $this->session->set_flashdata('errorMessage', 'Some Error Occurred While Updating Counselling Closure Status. Try Later.');
                redirect(current_url());
            }
        } else {
            redirect("admin/");
        }
    }

    public function editCounsellingClosure($aar_id) {
        if ($this->sessionvalidator->isLoggedIn()) {
            $viewData['counsellingClosureInfo'] = $this->CourseManagement->getCounsellingClosureInfoBy($aar_id)->result()[0];
            $viewData['courses'] = $this->CourseManagement->getNonDeletedCourses()->result();
            $this->load->view("admin/editCounsellingClosure", $viewData);
        } else {
            redirect("admin/");
        }
    }

    public function updateCounsellingClosure() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $aar_id = trim($this->input->post("aar_id"));
            $this->form_validation->set_rules('course', 'Course', 'trim|required', array('required' => 'Please Select Any Course.'));
            $this->form_validation->set_rules('scheduleStartDate', 'Start Date', 'trim|required', array('required' => 'Enter/Choose Start Date.'));
            $this->form_validation->set_rules('scheduleEndDate', 'End Date', 'trim|required', array('required' => 'Enter/Choose End Date.'));
            if ($this->form_validation->run() == FALSE) {
                $this->createCounsellingClosure();
            } else {
                $counsellingClosureUpdatedInfo = array(
                    'aar_id' => $aar_id,
                    'aar_start_date' => date("Y-m-d", strtotime(str_replace('/', '-', trim($this->input->post("scheduleStartDate"))))),
                    'aar_end_date' => date("Y-m-d", strtotime(str_replace('/', '-', trim($this->input->post("scheduleEndDate"))))),
                    'aar_updated_by' => $this->session->userdata("adminData")["smember_id"],
                    'aar_updated_on' => date("Y-m-d H:i:s")
                );
                if ($this->CourseManagement->updateCounsellingClosureInfo($counsellingClosureUpdatedInfo)) {
                    $this->session->set_flashdata('successMessage', 'Counselling Closure Info Updated Successfully.');
                    redirect("admin/Course/counsellingClosure");
                } else {
                    $this->session->set_flashdata('errorMessage', 'An Error Occured While Updating Counselling Closure Info. Try Later.');
                    redirect(current_url());
                }
            }
        } else {
            redirect("admin/");
        }
    }

}
