<?php

/**
 * Request Handler For Stock Issue & Return
 *
 * @author Softpro India Pvt. Ltd.
 */
class IssueAndReturn extends CI_Controller {

    public function __construct() {
        parent::__construct();
        $this->load->model('admin/InventoryManagement');
        $this->load->model('admin/StockAndPurchaseManagement');
        $this->load->model('admin/DepartmentManagement');
        $this->load->model('admin/LocationManagement');
        $this->load->model('admin/StaffManagement');
    }

    public function stockIssue() {
        if ($this->sessionvalidator->isLoggedIn() && $this->sessionvalidator->isAccessGranted()) {
            $viewData['purchaseHeads'] = $this->InventoryManagement->getNonDeletedActivePurchaseHeads()->result();
            $viewData['allPurchases'] = $this->StockAndPurchaseManagement->getAllPurchases()->result();
            $viewData['inventoryCategories'] = $this->InventoryManagement->getAllNonDeletedActiveInventoryCategories()->result();
            $viewData['allItems'] = $this->StockAndPurchaseManagement->getAllPurchaseItems()->result();
            $viewData['allNonIssuedStocks'] = $this->StockAndPurchaseManagement->getAvailableStocksForIssue()->result();
            $viewData['departments'] = $this->DepartmentManagement->getActiveNonDeletedDepartments()->result();
            $viewData['locations'] = $this->LocationManagement->getNonDeletedActiveLocations()->result();
            $viewData['selectedInvoice'] = "";
            $viewData['selectedCategory'] = "";
            $viewData['selectedAsset'] = "";
            if (isset($_POST['filterSubmitBtn'])) {
                $reqPurchaseHead = trim($this->input->post("pHead"));
                $reqInvoice = trim($this->input->post("invoice"));
                $reqInventoryCategory = trim($this->input->post("invCategory"));
                $reqAsset = trim($this->input->post("itemAssets"));
                $viewData['selectedInvoice'] = $reqInvoice;
                $viewData['selectedCategory'] = $reqInventoryCategory;
                $viewData['selectedAsset'] = $reqAsset;
                $viewData['allNonIssuedStocks'] = $this->StockAndPurchaseManagement->getAvailableStocksForIssue($reqPurchaseHead, $reqInvoice, $reqInventoryCategory, $reqAsset)->result();
                $this->load->view('admin/stockAndInventory/stockIssue', $viewData);
            } else {
                $this->load->view('admin/stockAndInventory/stockIssue', $viewData);
            }
        } else {
            redirect("admin/");
        }
    }

    public function saveNewStockIssue() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $selectedStock = $this->input->post('stockItem');
            $selectedDept = trim($this->input->post('empDept'));
            $selectedSubDept = trim($this->input->post('empSubDept'));
            $selectedLocation = trim($this->input->post('location'));
            $locationDetail = addslashes(trim($this->input->post('subLocation')));
            $selectedDate = date("Y-m-d", strtotime(str_replace('/', '-', $this->input->post('issueDate'))));
            $isEmpIssue = trim($this->input->post('empIssue'));
            $selectedEmployee = (trim($this->input->post('empIssue')) == "T") ? trim($this->input->post('employee')) : NULL;
            $newStockIssueToCreate = array();
            for ($i = 0; $i < sizeof($selectedStock); $i++) {
                array_push($newStockIssueToCreate, array(
                    'irm_issue_date' => $selectedDate,
                    'irm_is_emp_issue' => $isEmpIssue,
                    'irm_issued_by' => $this->session->userdata("adminData")["smember_id"],
                    'irm_issued_on' => date("Y-m-d H:i:s"),
                    'sm_id' => $selectedStock[$i],
                    'sub_dept_id' => $selectedSubDept,
                    'dept_id' => $selectedDept,
                    'smember_id' => $selectedEmployee,
                    'lm_id' => $selectedLocation,
                    'irm_location' => $locationDetail
                ));
            }
            $this->db->trans_start();
            for ($i = 0; $i < sizeof($newStockIssueToCreate); $i++) {
                $thisIssueId = $this->StockAndPurchaseManagement->createNewItemIssue($newStockIssueToCreate[$i]);
                $issueIdToUpdate = array(
                    'irm_id' => $thisIssueId,
                    'irm_issue_disp_id' => "IET0S" . $thisIssueId . "U"
                );
                $this->StockAndPurchaseManagement->updateIssueInfo($issueIdToUpdate);
            }
            $this->db->trans_complete();
            if ($this->db->trans_status() === FALSE) {
                $this->db->trans_rollback();
                $this->session->set_flashdata('errorMessage', 'An Error Occured While Issuing Item(s). Try Later.');
                redirect(current_url());
            } else {
                $this->db->trans_commit();
                $this->session->set_flashdata('successMessage', 'Item(s) Has Been Issued Successfully.');
                redirect("admin/IssueAndReturn/stockIssue");
            }
        } else {
            redirect("admin/");
        }
    }

    public function deleteIssue($irm_id) {
        if ($this->sessionvalidator->isLoggedIn()) {
            $this->StockAndPurchaseManagement->deleteIssueReturnInfo($irm_id);
            $this->session->set_flashdata('successMessage', 'Issue Record Deleted Successfully.');
            redirect("admin/Reports/issuedStock");
        } else {
            redirect("admin/");
        }
    }

    public function stockReturn() {
        if ($this->sessionvalidator->isLoggedIn() && $this->sessionvalidator->isAccessGranted()) {
            $viewData['purchaseHeads'] = $this->InventoryManagement->getNonDeletedActivePurchaseHeads()->result();
            $viewData['allPurchases'] = $this->StockAndPurchaseManagement->getAllPurchases()->result();
            $viewData['inventoryCategories'] = $this->InventoryManagement->getAllNonDeletedActiveInventoryCategories()->result();
            $viewData['allItems'] = $this->StockAndPurchaseManagement->getAllPurchaseItems()->result();
            $viewData['allIssuedStocks'] = $this->StockAndPurchaseManagement->issuedStocksReport()->result();
            $viewData['departments'] = $this->DepartmentManagement->getActiveNonDeletedDepartments()->result();
            $viewData['locations'] = $this->LocationManagement->getNonDeletedActiveLocations()->result();
            $viewData['selectedSubDepartment'] = "";
            $viewData['selectedEmployee'] = "";
            if (isset($_POST['filterSubmitBtn'])) {
                $reqPurchaseHead = trim($this->input->post("pHead"));
                $reqInvoice = trim($this->input->post("invoice"));
                $reqDepartment = trim($this->input->post("dept"));
                $reqSubDepartment = trim($this->input->post("subDept"));
                $reqEmployee = trim($this->input->post('emp'));
                $reqInventoryCategory = trim($this->input->post("invCategory"));
                $reqAssetItem = trim($this->input->post("itemAssets"));
                $reqLocation = trim($this->input->post("location"));
                $reqStartDate = (trim($this->input->post("startIssueDate")) == "") ? "" : date("Y-m-d", strtotime(str_replace('/', '-', trim($this->input->post("startIssueDate")))));
                $reqEndDate = (trim($this->input->post("endIssueDate")) == "") ? "" : date("Y-m-d", strtotime(str_replace('/', '-', trim($this->input->post("endIssueDate")))));
                $reqUIDOrSrNo = trim($this->input->post("uidOrSrNo"));
                $viewData['selectedSubDepartment'] = $reqSubDepartment;
                $viewData['selectedEmployee'] = $reqEmployee;
                $viewData['allIssuedStocks'] = $this->StockAndPurchaseManagement->issuedStocksReport($reqUIDOrSrNo, $reqPurchaseHead, $reqInvoice, $reqDepartment, $reqSubDepartment, $reqEmployee, $reqInventoryCategory, $reqLocation, $reqAssetItem, $reqStartDate, $reqEndDate)->result();
                $this->load->view('admin/stockAndInventory/stockReturn', $viewData);
            } else {
                $this->load->view('admin/stockAndInventory/stockReturn', $viewData);
            }
        } else {
            redirect("admin/");
        }
    }

    public function saveNewStockReturn() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $selectedStock = $this->input->post('selectForReturn');
            $stockReturnToUpdate = array();
            for ($i = 0; $i < sizeof($selectedStock); $i++) {
                echo $selectedStock[$i] . "o";
                array_push($stockReturnToUpdate, array(
                    'irm_id' => $selectedStock[$i],
                    'irm_return_date' => date("Y-m-d", strtotime(str_replace('/', '-', trim($this->input->post("returnDate"))))),
                    'irm_return_taken_by' => $this->session->userdata("adminData")["smember_id"],
                    'irm_returned_on' => date("Y-m-d H:i:s")
                ));
            }
            $this->db->trans_start();
            if (sizeof($stockReturnToUpdate)) {
                $this->StockAndPurchaseManagement->updateIssueInfoMulti($stockReturnToUpdate);
            }
            $this->db->trans_complete();
            $this->session->set_flashdata('successMessage', 'Item(s) Has Been Returned Successfully.');
            redirect("admin/IssueAndReturn/stockReturn");
        } else {
            redirect("admin/");
        }
    }

    /* Functions For Stock Verifications & Related Stuff */

    public function stockVerificationSetup() {
        if ($this->sessionvalidator->isLoggedIn() && $this->sessionvalidator->isAccessGranted()) {
            $viewData['verificationTitles'] = $this->StockAndPurchaseManagement->getAllVerificationTitles()->result();
            $this->load->view("admin/stockAndInventory/stockVerificationTitle", $viewData);
        } else {
            redirect("admin/");
        }
    }

    public function createVerificationTitle() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $this->load->view("admin/stockAndInventory/createStockVerificationTitle");
        } else {
            redirect("admin/");
        }
    }

    public function saveNewVerificationTitle() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $this->form_validation->set_rules('verificationTitle', 'Stock Verification Title', 'trim|required', array('required' => 'Stock Verification Title Is Required.'));
            $queryByTitleName = $this->StockAndPurchaseManagement->getVerificationTitleInfoByTitle(trim($this->input->post('verificationTitle')));
            $titleInfoByName = $queryByTitleName->result();
            if ($this->form_validation->run() == FALSE) {
                $this->createVerificationTitle();
            } else if (sizeof($titleInfoByName)) {
                $this->session->set_flashdata('errorMessage', "Thia Stock Verification Title With Already Exits. Please Choose A Different Name.");
                $this->createVerificationTitle();
            } else {
                $verificationFileNameWithPath = NULL;
                if (!empty($_FILES['verificationFile']['name'])) {
                    $studentImg['upload_path'] = './assets/admin/uploads/stockVerifications/';
                    $studentImg['allowed_types'] = 'pdf';
                    $studentImg['max_size'] = '6020';
                    $studentImg['file_ext_tolower'] = TRUE;
                    $studentImg['encrypt_name'] = TRUE;
                    $this->load->library('upload', $studentImg);
                    if (!$this->upload->do_upload('verificationFile')) {
                        $this->session->set_flashdata('errorMessage', "Scanned Verification File Can Not Be Uploaded As It Violates Upload File Criteria. File Size 5 MB Max. & Allowed Extension: Only *.pdf.");
                        $this->createVerificationTitle();
                    } else {
                        $verificationFileNameWithPath = "/assets/admin/uploads/stockVerifications/" . $this->upload->data()['file_name'];
                    }
                }
                $newStockVerificationInfo = array(
                    'svtm_name' => addslashes(trim($this->input->post('verificationTitle'))),
                    'svtm_description' => (trim($this->input->post('verificationTitleDesc')) == "") ? "NA" : addslashes(trim($this->input->post('verificationTitleDesc'))),
                    'svtm_verification_file' => $verificationFileNameWithPath,
                    'svtm_created_on' => date("Y-m-d H:i:s"),
                    'svtm_created_by' => $this->session->userdata("adminData")["smember_id"],
                    'svtm_updated_on' => date("Y-m-d H:i:s"),
                    'svtm_updated_by' => $this->session->userdata("adminData")["smember_id"]
                );
                if ($this->StockAndPurchaseManagement->createStockVerificationTitle($newStockVerificationInfo)) {
                    $this->session->set_flashdata('successMessage', 'Verification Title Created Successfully.');
                    redirect("admin/IssueAndReturn/stockVerificationSetup");
                } else {
                    $this->session->set_flashdata('errorMessage', 'An Error Occured While Creating Verification Title. Try Later.');
                    redirect(current_url());
                }
            }
        } else {
            redirect("admin/");
        }
    }

    public function deleteVerification($sv_id) {
        if ($this->sessionvalidator->isLoggedIn()) {
            $this->StockAndPurchaseManagement->deleteVerificationInfo($sv_id);
            $this->session->set_flashdata('successMessage', 'Verification Record Deleted Successfully.');
            redirect("admin/Reports/stockVerificationReport");
        } else {
            redirect("admin/");
        }
    }

    public function editVerificationTitle($svtm_id) {
        if ($this->sessionvalidator->isLoggedIn()) {
            $viewData['verificationTitleInfo'] = $this->StockAndPurchaseManagement->getVerificationTitleInfoBy($svtm_id)->result()[0];
            $this->load->view('admin/stockAndInventory/editStockVerificationTitle', $viewData);
        } else {
            redirect("admin/");
        }
    }

    public function updateStockVerificationTitle() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $svtm_id = $this->input->post('verificationTitleId');
            $this->form_validation->set_rules('verificationTitle', 'Stock Verification Title', 'trim|required', array('required' => 'Stock Verification Title Is Required.'));
            $verificationCurrentFileName = substr($this->input->post('currentFile'), strripos($this->input->post('currentFile'), "/") + 1);
            if ($this->form_validation->run() == FALSE) {
                $this->editVerificationTitle($svtm_id);
            } else if (!$this->StockAndPurchaseManagement->isVerificationTitleSafeUpdate($svtm_id, trim($this->input->post('verificationTitle')))) {
                $this->session->set_flashdata('errorMessage', "An Exam With This Name Already Exits. Please Choose A Different Name.");
                $this->editVerificationTitle($svtm_id);
            } else {
                $verificationFileNameWithPath = NULL;
                if (!empty($_FILES['verificationFile']['name'])) {
                    $studentImg['upload_path'] = './assets/admin/uploads/stockVerifications/';
                    $studentImg['allowed_types'] = 'pdf';
                    $studentImg['max_size'] = '6020';
                    $studentImg['file_ext_tolower'] = TRUE;
                    $studentImg['encrypt_name'] = TRUE;
                    $this->load->library('upload', $studentImg);
                    if (!$this->upload->do_upload('verificationFile')) {
                        $this->session->set_flashdata('errorMessage', "Scanned Verification File Can Not Be Uploaded As It Violates Upload File Criteria. File Size 5 MB Max. & Allowed Extension: Only *.pdf.");
                        $this->createVerificationTitle();
                    } else {
                        if (file_exists("./assets/admin/uploads/stockVerifications/" . $verificationCurrentFileName)) {
                            unlink("./assets/admin/uploads/stockVerifications/" . $verificationCurrentFileName);
                        }
                        $verificationFileNameWithPath = "/assets/admin/uploads/stockVerifications/" . $this->upload->data()['file_name'];
                    }
                } else {
                    $verificationFileNameWithPath = trim($this->input->post("currentInvoiceFile"));
                }
                $stockVerificationUpdateInfo = array(
                    'svtm_id' => $svtm_id,
                    'svtm_name' => addslashes(trim($this->input->post('verificationTitle'))),
                    'svtm_description' => (trim($this->input->post('verificationTitleDesc')) == "") ? "NA" : addslashes(trim($this->input->post('verificationTitleDesc'))),
                    'svtm_verification_file' => $verificationFileNameWithPath,
                    'svtm_updated_on' => date("Y-m-d H:i:s"),
                    'svtm_updated_by' => $this->session->userdata("adminData")["smember_id"]
                );
                if ($this->StockAndPurchaseManagement->updateStockVerificationInfo($stockVerificationUpdateInfo)) {
                    $this->session->set_flashdata('successMessage', 'Stock Verification Title Updated Successfully.');
                    redirect("admin/IssueAndReturn/stockVerificationSetup");
                } else {
                    $this->session->set_flashdata('errorMessage', 'An Error Occured While Updating Stock Verification Title. Try Later.');
                    redirect(current_url());
                }
            }
        } else {
            redirect("admin/");
        }
    }

    public function toggleVerificationTitleStatus($svtm_id, $toUpdateStatus) {
        if ($this->sessionvalidator->isLoggedIn()) {
            $verificationTitleUpdateData = array(
                'svtm_id' => $svtm_id,
                'svtm_updated_on' => date("Y-m-d H:i:s"),
                'svtm_updated_by' => $this->session->userdata("adminData")["smember_id"],
                'svtm_active_status' => $toUpdateStatus
            );
            if ($this->StockAndPurchaseManagement->updateStockVerificationInfo($verificationTitleUpdateData)) {
                $this->session->set_flashdata('successMessage', 'Stock Verification Title Updated Successfully.');
                redirect("admin/IssueAndReturn/stockVerificationSetup");
            } else {
                $this->session->set_flashdata('errorMessage', 'Some Error Occurred While Updating Stock Verification Title Status. Try Later.');
                redirect(current_url());
            }
        } else {
            redirect("admin/");
        }
    }

    public function deleteVerificationTitle($svtm_id) {
        if ($this->sessionvalidator->isLoggedIn()) {
            $verificationTitleUpdateData = array(
                'svtm_id' => $svtm_id,
                'svtm_updated_on' => date("Y-m-d H:i:s"),
                'svtm_updated_by' => $this->session->userdata("adminData")["smember_id"],
                'svtm_delete_status' => 'T'
            );
            if ($this->StockAndPurchaseManagement->updateStockVerificationInfo($verificationTitleUpdateData)) {
                $this->session->set_flashdata('successMessage', 'Stock Verification Title Deleted Successfully.');
                redirect("admin/IssueAndReturn/stockVerificationSetup");
            } else {
                $this->session->set_flashdata('errorMessage', 'Some Error Occurred While Deleting Stock Verification Title. Try Later.');
                redirect(current_url());
            }
        } else {
            redirect("admin/");
        }
    }

    public function undoDeleteVerificationTitle($svtm_id) {
        if ($this->sessionvalidator->isLoggedIn()) {
            $verificationTitleUpdateData = array(
                'svtm_id' => $svtm_id,
                'svtm_updated_on' => date("Y-m-d H:i:s"),
                'svtm_updated_by' => $this->session->userdata("adminData")["smember_id"],
                'svtm_delete_status' => 'F'
            );
            if ($this->StockAndPurchaseManagement->updateStockVerificationInfo($verificationTitleUpdateData)) {
                $this->session->set_flashdata('successMessage', 'Stock Verification Title Recovered Successfully.');
                redirect("admin/IssueAndReturn/stockVerificationSetup");
            } else {
                $this->session->set_flashdata('errorMessage', 'Some Error Occurred While Recovering Stock Verification Title. Try Later.');
                redirect(current_url());
            }
        } else {
            redirect("admin/");
        }
    }

    public function pendingStockVerificationPrint() {
        if ($this->sessionvalidator->isLoggedIn() && $this->sessionvalidator->isAccessGranted()) {
            $viewData['purchaseHeads'] = $this->InventoryManagement->getNonDeletedActivePurchaseHeads()->result();
            $viewData['allPurchases'] = $this->StockAndPurchaseManagement->getAllPurchases()->result();
            $viewData['inventoryCategories'] = $this->InventoryManagement->getAllNonDeletedActiveInventoryCategories()->result();
            $viewData['allItems'] = $this->StockAndPurchaseManagement->getAllPurchaseItems()->result();
            $viewData['stocksForVerification'] = array();
            $viewData['departments'] = $this->DepartmentManagement->getActiveNonDeletedDepartments()->result();
            $viewData['locations'] = $this->LocationManagement->getNonDeletedActiveLocations()->result();
            $viewData['employees'] = $this->StaffManagement->getAllWorkingEmployees()->result();
            $viewData['verificationTitle'] = $this->StockAndPurchaseManagement->getNonDeletedActiveVerificationTitles()->result();
            $viewData['selectedSubDepartment'] = "";
            $viewData['selectedEmployee'] = "";
            if (isset($_POST['filterSubmitBtn'])) {
                $reqVerificationTitle = trim($this->input->post('verificationTitle'));
                if ($reqVerificationTitle != "") {
                    $reqPurchaseHead = trim($this->input->post("pHead"));
                    $reqInvoice = trim($this->input->post("invoice"));
                    $reqDepartment = trim($this->input->post("dept"));
                    $reqSubDepartment = trim($this->input->post("subDept"));
                    $reqEmployee = trim($this->input->post('emp'));
                    $reqInventoryCategory = trim($this->input->post("invCategory"));
                    $reqAssetItem = trim($this->input->post("itemAssets"));
                    $reqLocation = trim($this->input->post("location"));
                    $reqStartDate = (trim($this->input->post("startIssueDate")) == "") ? "" : date("Y-m-d", strtotime(str_replace('/', '-', trim($this->input->post("startIssueDate")))));
                    $reqEndDate = (trim($this->input->post("endIssueDate")) == "") ? "" : date("Y-m-d", strtotime(str_replace('/', '-', trim($this->input->post("endIssueDate")))));
                    $reqUIDOrSrNo = trim($this->input->post("uidOrSrNo"));
                    $viewData['selectedSubDepartment'] = $reqSubDepartment;
                    $viewData['selectedEmployee'] = $reqEmployee;
                    $viewData['stocksForVerification'] = $this->StockAndPurchaseManagement->stocksForVerification($reqVerificationTitle, $reqUIDOrSrNo, $reqPurchaseHead, $reqInvoice, $reqDepartment, $reqSubDepartment, $reqEmployee, $reqInventoryCategory, $reqLocation, $reqAssetItem, $reqStartDate, $reqEndDate)->result();
                    $this->load->view('admin/stockAndInventory/stockVerificationPendingPrint', $viewData);
                } else {
                    $this->session->set_flashdata('errorMessage', "Please Select Verification Title To Fetch Item(s) For Verification.");
                    $this->load->view('admin/stockAndInventory/stockVerificationPendingPrint', $viewData);
                }
            } else {
                $this->load->view('admin/stockAndInventory/stockVerificationPendingPrint', $viewData);
            }
        } else {
            redirect("admin/");
        }
    }

    public function stockVerification() {
        if ($this->sessionvalidator->isLoggedIn() && $this->sessionvalidator->isAccessGranted()) {
            $viewData['purchaseHeads'] = $this->InventoryManagement->getNonDeletedActivePurchaseHeads()->result();
            $viewData['allPurchases'] = $this->StockAndPurchaseManagement->getAllPurchases()->result();
            $viewData['inventoryCategories'] = $this->InventoryManagement->getAllNonDeletedActiveInventoryCategories()->result();
            $viewData['allItems'] = $this->StockAndPurchaseManagement->getAllPurchaseItems()->result();
            $viewData['stocksForVerification'] = array();
            $viewData['departments'] = $this->DepartmentManagement->getActiveNonDeletedDepartments()->result();
            $viewData['locations'] = $this->LocationManagement->getNonDeletedActiveLocations()->result();
            $viewData['employees'] = $this->StaffManagement->getAllWorkingEmployees()->result();
            $viewData['verificationTitle'] = $this->StockAndPurchaseManagement->getNonDeletedActiveVerificationTitles()->result();
            $viewData['selectedSubDepartment'] = "";
            $viewData['selectedEmployee'] = "";
            if (isset($_POST['filterSubmitBtn'])) {
                $reqVerificationTitle = trim($this->input->post('verificationTitle'));
                if ($reqVerificationTitle != "") {
                    $reqPurchaseHead = trim($this->input->post("pHead"));
                    $reqInvoice = trim($this->input->post("invoice"));
                    $reqDepartment = trim($this->input->post("dept"));
                    $reqSubDepartment = trim($this->input->post("subDept"));
                    $reqEmployee = trim($this->input->post('emp'));
                    $reqInventoryCategory = trim($this->input->post("invCategory"));
                    $reqAssetItem = trim($this->input->post("itemAssets"));
                    $reqLocation = trim($this->input->post("location"));
                    $reqStartDate = (trim($this->input->post("startIssueDate")) == "") ? "" : date("Y-m-d", strtotime(str_replace('/', '-', trim($this->input->post("startIssueDate")))));
                    $reqEndDate = (trim($this->input->post("endIssueDate")) == "") ? "" : date("Y-m-d", strtotime(str_replace('/', '-', trim($this->input->post("endIssueDate")))));
                    $reqUIDOrSrNo = trim($this->input->post("uidOrSrNo"));
                    $viewData['selectedSubDepartment'] = $reqSubDepartment;
                    $viewData['selectedEmployee'] = $reqEmployee;
                    $viewData['stocksForVerification'] = $this->StockAndPurchaseManagement->stocksForVerification($reqVerificationTitle, $reqUIDOrSrNo, $reqPurchaseHead, $reqInvoice, $reqDepartment, $reqSubDepartment, $reqEmployee, $reqInventoryCategory, $reqLocation, $reqAssetItem, $reqStartDate, $reqEndDate)->result();
                    $this->load->view('admin/stockAndInventory/stockVerification', $viewData);
                } else {
                    $this->session->set_flashdata('errorMessage', "Please Select Verification Title To Fetch Item(s) For Verification.");
                    $this->load->view('admin/stockAndInventory/stockVerification', $viewData);
                }
            } else {
                $this->load->view('admin/stockAndInventory/stockVerification', $viewData);
            }
        } else {
            redirect("admin/");
        }
    }

    public function saveNewStockVerification() {
        if ($this->sessionvalidator->isLoggedIn()) {
            $selectedStock = $this->input->post('verifiedItems');
            $selectedStockStatus = $this->input->post('verifiedItemStatus');
            $selectedStockReasons = $this->input->post('verifiedItemReason');
            $stockVerificationToUpdate = array();
            for ($i = 0; $i < sizeof($selectedStock); $i++) {
                array_push($stockVerificationToUpdate, array(
                    'irm_id' => $selectedStock[$i],
                    'sv_date' => date("Y-m-d", strtotime(str_replace('/', '-', trim($this->input->post("verificationDate"))))),
                    'sv_entry_by' => $this->session->userdata("adminData")["smember_id"],
                    'sv_entry_date' => date("Y-m-d H:i:s"),
                    'sv_by_id' => (trim($this->input->post('verifyingEmployee')) == "") ? NULL : trim($this->input->post('verifyingEmployee')),
                    'sv_by_name' => addslashes(trim($this->input->post('verifyingPerson'))),
                    'sv_status' => trim($selectedStockStatus[$i]),
                    'sv_verify_quantity' => (trim($selectedStockStatus[$i]) == "NF") ? "0" : "1",
                    'sv_reason' => addslashes(trim($selectedStockReasons[$i])),
                    'svtm_id' => trim($this->input->post('verificationTitle'))
                ));
            }
            if (sizeof($stockVerificationToUpdate)) {
                $this->StockAndPurchaseManagement->createStockVerificationInfoMulti($stockVerificationToUpdate);
            }
            $this->session->set_flashdata('successMessage', 'Item(s) Verified Successfully.');
            redirect("admin/IssueAndReturn/stockVerification");
        } else {
            redirect("admin/");
        }
    }

}
