<?php

/**
 * Model For Handling All DB Operations Related To Sub-Departments
 *
 * @author Softpro India Pvt. Ltd.
 */
defined('BASEPATH') OR exit('No direct script access allowed');

class SubDepartmentManagement extends CI_Model {

    function createNewSubDepartment(array $newSubDepartmentInfo) {
        $this->db->insert('tbl_sub_departments_master', $newSubDepartmentInfo);
        return $this->db->insert_id();
    }

    function getAllSubDepartments() {
        $this->db->select("TSDM.sub_dept_id,TSDM.sub_dept_name,TSDM.sub_dept_description,TSDM.sub_dept_active_status,TSDM.sub_dept_delete_status,"
                . "TSDM.sub_dept_added_by,TSDM.sub_dept_added_on,TSDM.sub_dept_updated_by,TSDM.sub_dept_updated_on,TDM.dept_name,"
                . "CONCAT(TPRFLA.tprfl_firstname,' ',TPRFLA.tprfl_lastname) addedByAdmin,"
                . "CONCAT(TPRFLU.tprfl_firstname,' ',TPRFLU.tprfl_lastname) updatedByAdmin");
        $this->db->from('tbl_sub_departments_master TSDM');
        $this->db->join('tbl_department_master TDM', 'TSDM.dept_id = TDM.dept_id');
        $this->db->join('tbl_staff_members TSMA', 'TSDM.sub_dept_added_by = TSMA.smember_id');
        $this->db->join('tbl_profile TPRFLA', 'TSMA.smember_id = TPRFLA.tprfl_id');
        $this->db->join('tbl_staff_members TSMU', 'TSDM.sub_dept_updated_by = TSMU.smember_id');
        $this->db->join('tbl_profile TPRFLU', 'TSMU.smember_id = TPRFLU.tprfl_id');
        $this->db->order_by("TSDM.sub_dept_updated_on", "desc");
        return $this->db->get();
    }

    function getActiveNonDeletedSubDepartments() {
        $this->db->select("*");
        $this->db->from('tbl_sub_departments_master TSDM');
        $this->db->where("TSDM.sub_dept_active_status", "T");
        $this->db->where("TSDM.sub_dept_delete_status", "F");
        $this->db->order_by("TSDM.sub_dept_updated_on", "desc");
        return $this->db->get();
    }

    function getActiveNonDeletedSubDepartmentsUnderGivenDepartment($dept_id) {
        $this->db->select("*");
        $this->db->from('tbl_sub_departments_master TSDM');
        $this->db->where("TSDM.sub_dept_active_status", "T");
        $this->db->where("TSDM.sub_dept_delete_status", "F");
        $this->db->where("TSDM.dept_id", $dept_id);
        $this->db->order_by("TSDM.sub_dept_updated_on", "desc");
        return $this->db->get();
    }

    function getSubDepartmentInfoBy($sub_dept_id) {
        $this->db->select("TSDM.sub_dept_id,TSDM.sub_dept_name,TSDM.sub_dept_description,TSDM.sub_dept_active_status,TSDM.sub_dept_delete_status,"
                . "TSDM.sub_dept_added_by,TSDM.sub_dept_added_on,TSDM.sub_dept_updated_by,TSDM.sub_dept_updated_on,TSDM.dept_id,TDM.dept_name,"
                . "CONCAT(TPRFLA.tprfl_firstname,' ',TPRFLA.tprfl_lastname) addedByAdmin,"
                . "CONCAT(TPRFLU.tprfl_firstname,' ',TPRFLU.tprfl_lastname) updatedByAdmin");
        $this->db->from('tbl_sub_departments_master TSDM');
        $this->db->join('tbl_department_master TDM', 'TSDM.dept_id = TDM.dept_id');
        $this->db->join('tbl_staff_members TSMA', 'TSDM.sub_dept_added_by = TSMA.smember_id');
        $this->db->join('tbl_profile TPRFLA', 'TSMA.smember_id = TPRFLA.tprfl_id');
        $this->db->join('tbl_staff_members TSMU', 'TSDM.sub_dept_updated_by = TSMU.smember_id');
        $this->db->join('tbl_profile TPRFLU', 'TSMU.smember_id = TPRFLU.tprfl_id');
        $this->db->where('TSDM.sub_dept_id', $sub_dept_id);
        return $this->db->get();
    }

    function getSubDepartmentInfoByName($sub_dept_name) {
        $this->db->select("TSDM.sub_dept_id,TSDM.sub_dept_name,TSDM.sub_dept_description,TSDM.sub_dept_active_status,TSDM.sub_dept_delete_status,"
                . "TSDM.sub_dept_added_by,TSDM.sub_dept_added_on,TSDM.sub_dept_updated_by,TSDM.sub_dept_updated_on,TSDM.dept_id,TDM.dept_name,"
                . "CONCAT(TPRFLA.tprfl_firstname,' ',TPRFLA.tprfl_lastname) addedByAdmin,"
                . "CONCAT(TPRFLU.tprfl_firstname,' ',TPRFLU.tprfl_lastname) updatedByAdmin");
        $this->db->from('tbl_sub_departments_master TSDM');
        $this->db->join('tbl_department_master TDM', 'TSDM.dept_id = TDM.dept_id');
        $this->db->join('tbl_staff_members TSMA', 'TSDM.sub_dept_added_by = TSMA.smember_id');
        $this->db->join('tbl_profile TPRFLA', 'TSMA.smember_id = TPRFLA.tprfl_id');
        $this->db->join('tbl_staff_members TSMU', 'TSDM.sub_dept_updated_by = TSMU.smember_id');
        $this->db->join('tbl_profile TPRFLU', 'TSMU.smember_id = TPRFLU.tprfl_id');
        $this->db->where('TSDM.sub_dept_name', $sub_dept_name);
        return $this->db->get();
    }

    function isSubDepartmentNameSafeUpdate($sub_dept_id, $sub_dept_name) {
        $this->db->select("*");
        $this->db->from('tbl_sub_departments_master');
        $this->db->where('sub_dept_name', $sub_dept_name);
        $this->db->where('sub_dept_id != ' . $sub_dept_id);
        $result = $this->db->get()->result();
        if (sizeof($result)) {
            return FALSE;
        } else {
            return TRUE;
        }
    }

    function updateSubDepartmentInfo(array $subDepartmentUpdatedInfo) {
        $this->db->where('sub_dept_id', $subDepartmentUpdatedInfo['sub_dept_id']);
        return $this->db->update('tbl_sub_departments_master', $subDepartmentUpdatedInfo);
    }

}
